/******************************************************************************
 *            Copyright (c) 2006 Michel Gutierrez. All Rights Reserved.
 ******************************************************************************/

function MediaResp() {
	this.entries={};
	this.updateDone=false;
	var prefBranch2=DWHelper.pref.QueryInterface(Components.interfaces.nsIPrefBranch2);
	prefBranch2.addObserver("", this, false);
	this.updateReqExtensions();
	this.updateMediaWeight();
	this.typePattern=new RegExp("^(audio|video)/");
	this.ytSigPattern=/^(http:\/\/(?:[^\/]*youtube\..*|.*origin=[^\/&]*youtube\..*))(?:&|&amp;)signature.*(?:&|&amp;)ip=.*$/;
	
	this.cacheService = Components.classes["@mozilla.org/network/cache-service;1"]
		.getService(Components.interfaces.nsICacheService);
	this.httpCacheSession = this.cacheService.createSession("HTTP", 
		Components.interfaces.nsICache.STORE_ANYWHERE,
		Components.interfaces.nsICache.STREAM_BASED);
	this.httpCacheSession.doomEntriesIfExpired=false;
	
	this.entryTracker={};
}

MediaResp.prototype.updateReqExtensions=function() {
	var exts="flv|ram|mpg|mpeg|avi|rm|wmv|mov|asf|mp3|rar|movie|divx";
	try {
		exts=DWHelper.pref.getCharPref("mediareq-extensions");
	} catch(e) {
	}
	this.reqPattern=new RegExp("[/\\?&]([^/\\?&=]+\\.("+exts+"))(?:$|\\?|&|/)");
}

MediaResp.prototype.updateMediaWeight=function() {
	this.mediaWeightEnabled=false;
	try {
		this.mediaWeightEnabled=DWHelper.pref.getBoolPref("mediaweight-enabled");
	} catch(e) {}
	this.mediaWeightThreshold=1048576;
	try {
		this.mediaWeightThreshold=DWHelper.pref.getIntPref("mediaweight-threshold");
	} catch(e) {}
}

MediaResp.prototype.observe=function(subject,topic,data) {
	if(topic=="nsPref:changed") {
		if(data=="mediareq-extensions")
			this.updateReqExtensions();
		if(data=="mediaweight-enabled" || data=="mediaweight-threshold")
			this.updateMediaWeight();
	}
}

MediaResp.prototype.cleanEntries=function() {
	var entries={};
	var timeNow=new Date().getTime();
	for(var i in this.entries) {
		var entry=this.entries[i];
		if(timeNow-entry.time<DWHelper_getMenuHttpExpiration()) {
			entries[i]=entry;
		}
	}
	this.entries=entries;
}

MediaResp.prototype.updateMenu=function(menu) {
	this.cleanEntries();
	var urlAsTooltip=true;
	try {
		urlAsTooltip=DWHelper.pref.getBoolPref("url-as-tooltip");
	} catch(e) {}
	for(var i in this.entries) {
		var entry=this.entries[i];
		
		var filename=DWHelper_getConvertedFileName(entry.filename,entry.format);
		
		var data={
			controler: "mediaresp", 
			src: "chrome://dwhelper/skin/mediaresp.gif",
			label: filename,
			value: ""+i
		};
		if(urlAsTooltip) 
			data['tooltiptext']=entry.url;
		menu.push(data);
	}
	this.updateDone=true;
}

MediaResp.prototype.download=function(value,key) {
	//dump("MediaResp.prototype.download("+value+","+copyLink+")\n");
	var entry=this.entries[value];
	if(entry!=null) {
		if(key==DWHelper.KEY_COPYURL) {
			DWHUtil.toClipboard(entry.url);
		} else {
			var format=entry.format;
			if(key==DWHelper.KEY_REQCONVERT && DWHelper_isConverterEnabled()) {
				format=DWHelper_askFormat(format);
			}
			var filename=DWHelper_getUniqueFile("mediaresp",entry.filename);
			if(DWHelper_canChooseName()) {
				try {
					if(key==DWHelper.KEY_QUICK) {
						DWHelper_downloadFileQuick(filename,entry.url,entry.pageUrl,format);
					} else {
						DWHelper_downloadFileAs(filename,entry.url,entry.pageUrl,format);
					}
				} catch(e) { }
			} else {
				try {
					DWHelper_downloadFile("mediaresp",filename,entry.url,entry.pageUrl,format);
				} catch(e) { }
			}
		}
   	}
}

MediaResp.prototype.reset=function() {
}

MediaResp.prototype.newResponse=function(subject) {
    var request=subject.QueryInterface(Components.interfaces.nsIRequest);

	var murl=request.name;
    var httpChannel=subject.QueryInterface(Components.interfaces.nsIHttpChannel);
    
	var location=null;
	try {
		location=httpChannel.getResponseHeader("location");
	} catch(e) {}
	if(location) {
		if(this.entries[murl]) {
			delete this.entries[murl];
		}
		return;
	}
    
	var contentType=null;
	try {
		contentType=httpChannel.getResponseHeader("content-type");
	} catch(e) {}
	var contentLength=null;
	try {
		contentLength=httpChannel.getResponseHeader("content-length");
	} catch(e) {}
	var contentDisp=null;
	try {
		contentDisp=httpChannel.getResponseHeader("content-disposition");
	} catch(e) {}
	
	if(contentLength!=null) {
	
		var tms="100";
		try {
			tms=DWHelper.pref.getCharPref("trigger-min-size");
		} catch(e) {}
		tms=parseFloat(tms);
		if(!isNaN(tms)) {
			if(contentLength<tms*1024)
				return;
		}
	}

	var wnd=null;
	if(httpChannel.loadGroup && httpChannel.loadGroup.notificationCallbacks)
		wnd=httpChannel.loadGroup.notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow);
	else if(httpChannel.notificationCallbacks)
		wnd=httpChannel.notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow);

	/*
	var shortenYTAddress=true;
	try {
		shortenYTAddress=DWHelper.pref.getBoolPref("shorten-yt-address");
	} catch(e) {}
	if(shortenYTAddress) {
		if(this.ytSigPattern.test(murl)) {
			var m=this.ytSigPattern.exec(murl);
			murl=m[1];
		}
	}
	*/

	var filename=this.analyzeMeta(murl,contentType,contentDisp,contentLength,wnd);
	if(filename!=null) {

		var forceCaching=true;
		try {
			forceCaching=DWHelper.pref.getBoolPref("force-cache");
		} catch(e) {}
		
		if(forceCaching) {
			httpChannel.setResponseHeader("Cache-Control","max-age="+24*60*60,false);
		}

	    var referer=null;
		try {
			referer=request.getRequestHeader("referer");
		} catch(e) {
		}
		try {
			if(filename.substr(-4)==".flv") {
				var wnd=null;
				try {
					wnd=httpChannel.notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow);
				} catch(e) {
				}
				var url=null;
				if(wnd!=null && wnd.document!=null)
					url=wnd.document.URL;
				if(!/^http:\/\/[^\/]*downloadhelper.net\/watch\.php/.test(murl)) {
					DWHelper.listMgr.addToList("http://downloadhelper.net/1.0#history-list",
						murl,"flv",url,filename,referer);
				}
			}
		} catch(e) {
		}
	}
}

MediaResp.prototype.analyzeMeta=function(murl,contentType,contentDisp,contentLength,wnd) {
    
    var hit=false;
    var filename=null;
  
	if(contentType!=null && this.typePattern.test(contentType)) {
		var excludeAsf=true;
		try {
			excludeAsf=DWHelper.pref.getBoolPref("exclude-ms-asf");
		} catch(e) {}
		if(excludeAsf) {
			if(!/ms-asf$/.test(contentType))
				hit=true;
		} else {
			hit=true;
		}
	}
	
	if(!hit) {
		if(this.reqPattern.test(murl)) {
			hit=true;
		}
	}
	
	if(hit==false && this.mediaWeightEnabled==true) {
		try {
			if(contentLength!=null && isNaN(contentLength)==false && contentLength>=this.mediaWeightThreshold) {
				hit=true;
			}	
		} catch(e) {
		}
	}

	if(hit) {
	
		filename=this.guessFileName(murl,contentType,contentDisp,wnd);
    
		try {
	
		if(filename.length>64) {
			var parts=/^(.*)(\..*?)$/.exec(filename);
			filename=parts[1].substr(0,64-parts[2].length)+parts[2];
		}

		this.entries[murl]={
			url: murl, filename: filename,
			time: new Date().getTime()
			};

		var pageUrl=null;
		if(wnd!=null && wnd.document) {
			this.entries[murl].pageUrl=wnd.document.URL;
			pageUrl=wnd.document.URL;
    	}
			
		this.entries[murl].format=DWHelper_getDefaultFormat(filename,murl,pageUrl);
		
		if(this.updateDone==true) {
			DWHelper_loadMenu();
		}

		} catch(e) {
		}

		return filename;
	}
	return null;
}

MediaResp.prototype.guessFileName=function(murl,contentType,contentDisp,wnd) {

	var filename=null;

	if(contentDisp!=null) {
		if(/filename=/.test(contentDisp)) {
			filename=/filename="?([^;"]*)/.exec(contentDisp)[1];
		}
	}
	if(filename==null) {
		var extension=null;
		if(contentType!=null && /^video\/x-.*$/.test(contentType)) {
			extension=/video\/x-([^;]*)/.exec(contentType)[1];
		} else if(contentType!=null && DWHUtil.startsWith(contentType,"video/")) {
			extension=/video\/([^ ,]*).*$/.exec(contentType)[1];
		} else if(contentType!=null && DWHUtil.startsWith(contentType,"audio/")) {
			extension=/audio\/(?:x-)?([^ ,]*).*?$/.exec(contentType)[1];
		} else {
			extension="flv";
		}
		var re=new RegExp("([^/&\\?]+\\."+extension+")(?:$|&|\\?)");
		if(re.test(murl)) {
			var m=re.exec(murl);
			filename=m[1];
		} else if(this.reqPattern.test(murl)) {
			var m=this.reqPattern.exec(murl);
			filename=m[1];
			extension=m[2];
		} else {
			try {
				var title=null;
				if(wnd) {
					title=DWHUtil.xpGetString(wnd.document.documentElement,
						"/html/head/meta[@name='title']/@content");
					if(title==null || title=="")
						title=DWHUtil.xpGetString(wnd.document.documentElement,
							"/html/head/title");
				}
				if(title==null || title=="")
					title="file-"+Math.floor(Math.random()*1000000000);
				filename=title.replace(/[^a-zA-Z0-9-]+/g,"_")+"."+extension;
			} catch(e) {
				filename="file-"+Math.floor(Math.random()*1000000000)+"."+extension;
			}
		}
	}
	return filename;
}

MediaResp.prototype.checkCacheTracker=function(url,wnd) {
	try {
		var cacheEntryDescriptor=this.httpCacheSession.openCacheEntry(url, 
					Components.interfaces.nsICache.ACCESS_READ, false);
		if(cacheEntryDescriptor && (cacheEntryDescriptor.accessGranted & 1)) {			
				var headers=cacheEntryDescriptor.getMetaDataElement("response-head");
				if(/Location:/i.test(headers)) {
					cacheEntryDescriptor.close();
					return;
				}
	
				var contentType=null;
				try {
					contentType=/Content-Type: *(.*)/i.exec(headers)[1];
				} catch(e) {}
				var contentLength=null;
				try {
					contentLength=/Content-Length: *(.*)/i.exec(headers)[1];
				} catch(e) {}
				var contentDisp=null;
				try {
					contentDisp=/Content-Disposition: *(.*)/i.exec(headers)[1];
				} catch(e) {}
				
				var fn=this.analyzeMeta(url,contentType,contentDisp,contentLength,wnd);
				if(fn!=null) {
					//dump("[MediaResp] checkCacheTracker("+url+"): hit\n");
				}
				cacheEntryDescriptor.close();
			}
	} catch(e) {}		
}

MediaResp.prototype.newRequest=function(url,request) {
	try {

		var cacheTracker=true;
		try {
			cacheTracker=DWHelper.pref.getBoolPref("cache-tracker");
		} catch(e) {}
		if(cacheTracker) {

		    var httpChannel=request.QueryInterface(Components.interfaces.nsIHttpChannel);
			var wnd=null;
			if(httpChannel.notificationCallbacks)
				wnd=httpChannel.notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow);
			else if(httpChannel.loadGroup && httpChannel.loadGroup.notificationCallbacks)
				wnd=httpChannel.loadGroup.notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow);

			function checkCacheTracker(url,mediaresp,wnd) {
				mediaresp.checkCacheTracker(url,wnd);			
			}
			setTimeout(checkCacheTracker,0,url,this,wnd);
		}
			
	} catch(e) { }
}

MediaResp.prototype.quickDownload=function(url) {
	for(var i in this.entries) {
		var entry=this.entries[i];
		if(url==entry.pageUrl) {
			var filename=DWHelper_getUniqueFile("mediaresp",entry.filename);
			var format=entry.format;
			DWHelper_downloadFileQuick(filename,entry.url,entry.pageUrl,format);
			return true;
		}
	}
	return false;
}

